@@ -10,9 +10,9 @@ import glob
 from sklearn.preprocessing import MinMaxScaler, StandardScaler
 
 # Micah Yarbrough
-# 10/4/25
+# 11/7/25
 # This function will extract and clean up data from .xlsx files, saving them as .csv's
-def data_cleaner(filepath, savepath, overwrite = False, skip = False, varspath = "vars_of_interest.json"):
+def data_cleaner(filepath, savepath, overwrite= False, skip= False, varspath= "vars_of_interest.json", downsample= True):
     """
     Preprocesses .xlsx files into fennec question-usefull .csv files.
 
@@ -22,6 +22,7 @@ def data_cleaner(filepath, savepath, overwrite = False, skip = False, varspath =
         overwrite (bool): Skips the overwrite checker if true.
         skip (bool): Skips duplicate files instead of checking or overwriting if true.
         varspath (string): The vars-of-interest.json path. Defaults to same folder as THIS script.
+        downsample (bool): If true, it will downsample to the lowest sample rate, if false it will upsample to the highest
 
     Relies on the vars_of_interest.json file to determine what data is wanted
     """
@@ -91,6 +92,8 @@ def data_cleaner(filepath, savepath, overwrite = False, skip = False, varspath =
 
     extracted_data = {key: None for key in vars_of_interest} #stores only the designated data from each xl sheet
 
+    sheets = vars_of_interest.keys()
+
     #get the correct data from each sheet in the pandas array
     for sheet, variables in vars_of_interest.items():
         #make sure sheet exist in .xlsx
@@ -112,15 +115,35 @@ def data_cleaner(filepath, savepath, overwrite = False, skip = False, varspath =
 
         extracted_data[sheet] = df[variables].to_numpy(dtype=float) #save the designated data to extracted_data as a numpy array
         
-        #FREQUENCY CORRECTION
-        if(sheet == "RCOU" or sheet == "RCIN"):
-            extracted_data[sheet] = np.repeat(extracted_data[sheet], 40, axis=0).astype(float) #IMU freq. / RCOU/IN freq. = 400Hz / 10Hz = 40
+    #FREQUENCY CORRECTION    
+    lengths = [len(arr) for arr in extracted_data.values()] # get total timesteps for each sheet
+    min_len = min(lengths) # find shortest sheet
+    ratios = [round(l / min_len) for l in lengths]
 
+    # Scale each sheet according to its scaling ratio
+    for i, (sheet_name, sheet) in enumerate(extracted_data.items()):
+        ratio = ratios[i]
+        sheet = sheet[::ratio]
+        # Downsample
+        if downsample == True and ratio != 1:
+            extracted_data[sheet_name] = sheet  # reassign if you want to keep the change
+            print(f"Sheet {sheet_name}, downsampled by {ratio}")
+        # Upsample
+        elif downsample == False:
+            ratio = max(ratios)/ratio
+            if ratio != 1:
+                extracted_data[sheet_name] = np.repeat(extracted_data[sheet_name], max(ratios)/ratio, axis=0).astype(float)
+                print(f"Sheet {sheet_name}, upsampled by {ratio}")
+
+    # Recalculate the lengths
+    lengths = [len(arr) for arr in extracted_data.values()]
+    min_len = min(lengths)
+ 
     # --- LENGTH CORRECTION ---
-    min_len = min(len(arr) for arr in extracted_data.values()) # Find the minimum length among all the np arrays
     #Truncate all arrays to the minimum length
     for sheet in extracted_data: 
         extracted_data[sheet] = extracted_data[sheet][:min_len] 
+    lengths = [len(arr) for arr in extracted_data.values()]
 
     #stack all the data from each sheet into one single 2D array
     csv_data = np.hstack(list(extracted_data.values()))
@@ -131,10 +154,13 @@ def data_cleaner(filepath, savepath, overwrite = False, skip = False, varspath =
     df.to_csv(new_path, index=False, encoding='utf_8') # Save to new path
 
     print(f"{inputfile} processed and saved to {savepath} as {filename}.csv")
+    xl.close()
     return True
 
-
-def folder_cleaner(excel_dir, savepath, overwrite = False, skip = False, varspath = "vars_of_interest.json"):
+# Micah Yarbrough and Wills Kookogey
+# 10/21/25
+# This function will calls the data cleaner for every .xlsx file in a given directory
+def folder_cleaner(excel_dir, savepath, overwrite = False, skip = False, varspath = "vars_of_interest.json", downsample= True):
     """
     Preprocesses a folder of .xlsx files into fennec question-usefull .csv files.
 
@@ -159,7 +185,8 @@ def folder_cleaner(excel_dir, savepath, overwrite = False, skip = False, varspat
     # for each file in the folder, run data_cleaner
     for file in os.listdir(excel_dir):
         filepath = os.path.join(excel_dir, file)
-        data_cleaner(filepath, savepath, overwrite, skip, varspath)
+        if filepath.lower().endswith(".xlsx"):
+            data_cleaner(filepath, savepath, overwrite, skip, varspath, downsample)
 
 
 
@@ -280,10 +307,10 @@ def standardize(csv_dir, weights = [None], offsets = [None], scaler= StandardSca
 
 # Micah Yarbrough 
 # 10/9/25
-# Segments, labels, and sorts data into dataset dictionary
-def get_CG_labels(csv_dir):
+# Reads all filenames in a folder and returns 1D CG characterization labels
+def get_1D_CG_labels(csv_dir):
     """
-    Reads all filenames in a folder and returns characterization labels
+    Reads all filenames in a folder and returns 1D CG characterization labels
 
     Args:
         csv_dir (string): Directory of .csv files from which to get labels
@@ -330,6 +357,74 @@ def get_CG_labels(csv_dir):
     return labels
 
 
+# Micah Yarbrough 
+# 10/17/25
+# Reads all filenames in a folder and returns 2D CG characterization labels
+def get_2D_CG_labels(csv_dir):
+    """
+    Reads all filenames in a folder and returns 2D CG characterization labels
+
+    Args:
+        csv_dir (string): Directory of .csv files from which to get labels
+    
+    Returns:
+        labels (list): A list of all the characterization labels
+
+    """
+    # the output labels list
+    labels = []
+
+    # Regex patterns for reading 2024-2025 1D and 2D CG flight data files
+    patternAAP = r'^\d+G_(AAP)_(L|H|S)_\d+\.csv$'
+    patternAAC = r'^\d+G_(AAC)_(L|H|S)_\d+\.csv$'
+    patternAAS = r'^\d+G_(AAS)_(L|H|S)_\d+\.csv$'
+
+    patternBBP = r'^\d+G_(BBP)_(L|H|S)_\d+\.csv$'
+    patternBBC = r'^\d+G_(BBC)_(L|H|S)_\d+\.csv$'
+    patternBBS = r'^\d+G_(BBS)_(L|H|S)_\d+\.csv$'
+
+    patternCCP = r'^\d+G_(CCP)_(L|H|S)_\d+\.csv$'
+    patternCCC = r'^\d+G_(CCC)_(L|H|S)_\d+\.csv$'
+    patternCCS = r'^\d+G_(CCS)_(L|H|S)_\d+\.csv$'
+
+    # --- CHECK FILEPATH ---
+    if not os.path.isdir(csv_dir): # does savepath exist?
+        raise FileNotFoundError(
+            f"Error: Save path '{csv_dir}' not found. "
+            f"Please create the directory before running the function."
+        )
+
+    # --- GET FILEPATHS ---
+    csv_files = sorted(glob.glob(os.path.join(csv_dir, "*.csv")))
+    # SORTED() IS ESSENTIAL TO ENSURE FILES MATCH normalize() data
+
+    # append label of each file to labels list
+    for file in csv_files:
+        filename = os.path.basename(file)
+        if re.search(patternAAP, filename):
+            labels.append("AAP")
+        if re.search(patternAAC, filename):
+            labels.append("AAC")
+        if re.search(patternAAS, filename):
+            labels.append("AAS")
+
+        if re.search(patternBBP, filename):
+            labels.append("BBP")
+        if re.search(patternBBC, filename):
+            labels.append("BBC")
+        if re.search(patternBBS, filename):
+            labels.append("BBS")
+
+        if re.search(patternCCP, filename):
+            labels.append("CCP")
+        if re.search(patternCCC, filename):
+            labels.append("CCC")
+        if re.search(patternCCS, filename):
+            labels.append("CCS")
+    
+    return labels
+
+
 # Wills Kookogey & Micah Yarbrough
 # 10-09-25
 # This function segments the data and splits it into train/validate/test.
@@ -337,6 +432,7 @@ def get_CG_labels(csv_dir):
 def segment_and_split(input_data, input_labels, timesteps, train_split=0.7, validate_split=0.15):
     """
     Segments, labels, and sorts data into dataset dictionary
+    output shape is (batch, sequence, feature)
 
     Args:
         input_data (list): List of numpy arrays, 1 per file
@@ -350,7 +446,7 @@ def segment_and_split(input_data, input_labels, timesteps, train_split=0.7, vali
             each set has the follwing labels: "sets" and "labels" 
                 - "sets" : list of sets, corresponds to "labels"
                 - "labels" : list of labels, corresponds to "sets"
-            ex: output["Training_Set"]["sets"]
+            ex: output["Training_Set"]["sets"] 
 
     """
     # --- ERROR CHECKS ---
